uniform mat4 uMVPMatrix; //总变换矩阵
uniform mat4 uMMatrix; //变换矩阵
uniform vec3 uLightLocation;//光源位置
uniform vec3 uCamera;//摄像机位置

attribute vec3 aPosition;  //顶点位置
attribute vec3 aNormal;    //顶点法向量
attribute vec2 aTextureCoord;   //顶点纹理坐标

varying vec2 vTextureCoord;  //用于传递给片元着色器的变量
varying vec4 vambient;
varying vec4 vdiffuse;
varying vec4 vspecular;

//定位光光照计算的方法
void pointLight(//定位光光照计算的方法
  in vec3 normal,//法向量
  inout vec4 ambient,//环境光最终强度
  inout vec4 diffuse,//散射光最终强度
  inout vec4 specular,//镜面光最终强度
  in vec3 lightLocation,//光源位置
  in vec4 lightAmbient,//环境光强度
  in vec4 lightDiffuse,//散射光强度
  in vec4 lightSpecular//镜面光强度
){
  ambient=lightAmbient;//直接得出环境光的最终强度

  vec3 normalTarget=normal;//计算变换后的法向量
  vec3 newNormal=(uMMatrix*vec4(normalTarget,1)).xyz;
  newNormal=normalize(newNormal); //对法向量规格化
  vec3 eye= normalize(uCamera-(uMMatrix*vec4(aPosition,1)).xyz);   //计算从表面点到摄像机的向量
  vec3 vp= normalize(lightLocation-(uMMatrix*vec4(aPosition,1)).xyz);  //计算从表面点到光源位置的向量vp
  float nDotViewPosition=max(0.0,dot(newNormal,vp));

  diffuse=lightDiffuse*nDotViewPosition;//计算散射光的最终强度

  vec3 halfVector=normalize(vp+eye);//求视线与光线的半向量
  float shininess=50.0;//粗糙度，越小越光滑
  vec3 Rvec=normalize(2.0*halfVector*newNormal-vp);
  float nDotViewHalfVector=dot(Rvec,eye);//法线与半向量的点积
  float powerFactor=max(0.0,pow(nDotViewHalfVector,shininess));//镜面反射光强度因子

  specular=lightSpecular*powerFactor;//计算镜面光的最终强度

}

void main()
{
   gl_Position = uMVPMatrix * vec4(aPosition,1); //根据总变换矩阵计算此次绘制此顶点位置
   pointLight(normalize(aNormal),vambient,vdiffuse,vspecular,uLightLocation,vec4(0.8,0.8,0.8,1.0),vec4(0.7,0.7,0.7,1.0),vec4(0.3,0.3,0.3,1.0));
   vTextureCoord = aTextureCoord;//将接收的纹理坐标传递给片元着色器
}
